Gfer := module()
	option package;
  description "Corrections applied to AGA instruments";
	export Ell_Req, f_id, d_t, d_v, I_t,df_th, g_th, g_b, df_mic;
  export Cp_Cu, K_Cu, d_sk,df_skTE, df_skTM1n, df_skTE1n, df_dcTM1n, df_dcTE, Eps_TM1n, Eps_TE1n;
  export dzm2_zTE1n, dzm2_zTM1n, df_ducts, dz2_z2AC;
	export df_dth, df_Ith, df_CuTh, g_t, g_thO2, g_Cu;

  (* local	libGfer;*)

(* USE gfer.so for LINUX and gfer.dll for WINDOWS *)
(* libGfer := "gfer.so" *)

Ell_Req := define_external('Ell_Req', LIB=libGfer, 
			 'Rx'		::REF(float[8]),
			 'Ry'		::REF(float[8]),
			 'Rz'		::REF(float[8]), RETURN::(float[8]));

f_id	 := define_external('f_id', LIB=libGfer, 
			 'n'		::REF(integer[4]),
			 'w'		::REF(float[8]),
			 'Req'	::REF(float[8]), RETURN::(float[8]));

d_t		 := define_external('d_t', LIB=libGfer, 
			 'K'		::REF(float[8]),
			 'Rho'	::REF(float[8]),
			 'Cp'		::REF(float[8]), 
			 'f'		::REF(float[8]), RETURN::(float[8]));

d_v		 := define_external('d_v', LIB=libGfer, 
			 'Nu'		::REF(float[8]),
			 'Rho'	::REF(float[8]),
			 'f'		::REF(float[8]), RETURN::(float[8]));

I_t		 := define_external('I_t', LIB=libGfer, 
			 'T'		::REF(float[8]),
			 'p'		::REF(float[8]),
			 'Cv'		::REF(float[8]), 
			 'K'		::REF(float[8]),
			 'm0'		::REF(float[8]),
			 'h'		::REF(float[8]), RETURN::(float[8]));

df_th	 := define_external('df_th', LIB=libGfer, 
			 'g'		::REF(float[8]),
			 'dt'		::REF(float[8]),
			 'dt_Cu'::REF(float[8]), 
			 'I_th'	::REF(float[8]),
			 'K_gas'::REF(float[8]),
			 'K_Cu'	::REF(float[8]), 
			 'Req'	::REF(float[8]),
			 'f'		::REF(float[8]), RETURN::(float[8]));

df_dth := define_external('df_dth', LIB=libGfer, 
			 'g'		::REF(float[8]),
			 'Req'	::REF(float[8]),
			 'dt'		::REF(float[8]), RETURN::(float[8]));

df_Ith := define_external('df_Ith', LIB=libGfer, 
			 'g'		::REF(float[8]),
			 'Req'	::REF(float[8]),
			 'I_th'	::REF(float[8]), RETURN::(float[8]));

df_CuTh := define_external('df_CuTh', LIB=libGfer, 
			 'g'		::REF(float[8]),
			 'Req'	::REF(float[8]),
			 'dt_Cu'::REF(float[8]),
			 'K_gas'::REF(float[8]),
			 'K_Cu'	::REF(float[8]), RETURN::(float[8]));

g_t 	 := define_external('g_t', LIB=libGfer, 
			 'T'		::REF(float[8]),
			 'p'		::REF(float[8]),
  		 'g'		::REF(float[8]),	
			 'dt_th'::REF(float[8]), 
			 'dt_Cu'::REF(float[8]), 
			 'K_gas'::REF(float[8]),			
			 'K_Cu'	::REF(float[8]), 
			 'Req'	::REF(float[8]),
			 'f'		::REF(float[8]), RETURN::(float[8]));

g_th 	 := define_external('g_th', LIB=libGfer, 
			 'g'		::REF(float[8]),	
			 'Req'	::REF(float[8]),
			 'dt_th'::REF(float[8]), RETURN::(float[8]));

g_thO2 := define_external('g_thO2', LIB=libGfer, 
			 'g'		::REF(float[8]),	
			 'Req'	::REF(float[8]),
			 'dt_th'::REF(float[8]), RETURN::(float[8]));

g_Cu 	 := define_external('g_Cu', LIB=libGfer, 
			 'g'		::REF(float[8]),	
			 'Req'	::REF(float[8]),
			 'dt_Cu'::REF(float[8]), 
			 'K_gas'::REF(float[8]),			
			 'K_Cu'	::REF(float[8]), RETURN::(float[8]));

g_b	  := define_external('g_b', LIB=libGfer, 
			 'T'		::REF(float[8]),
			 'p'		::REF(float[8]),
  		 'w'		::REF(float[8]),	
			 'g'		::REF(float[8]), 
			 'd_v'  ::REF(float[8]), 
			 'd_th' ::REF(float[8]),			
			 'f'		::REF(float[8]), RETURN::(float[8]));

df_ducts := define_external('df_ducts', LIB=libGfer,
			'r'			::ARRAY(1..N, float[8]),
 			'L'			::ARRAY(1..N, float[8]),
			'n'			::REF(integer[4]),
			'rho'		::REF(float[8]),
			'cp'		::REF(float[8]),
			'w'			::REF(float[8]),
			'g'			::REF(float[8]),
			'K_g'		::REF(float[8]),
			'Nu_g'	::REF(float[8]),
			'Req'		::REF(float[8]),
			'f'			::REF(float[8]),
			'nn'		::REF(integer[4]),
			'df_f'	::REF(float[8]),
			'dg_f'	::REF(float[8]));


df_mic := define_external('df_mic', LIB=libGfer, 
			 'r'		::REF(float[8]),
			 'Req'	::REF(float[8]),
  		 'Rho'	::REF(float[8]),	
			 'w'		::REF(float[8]), RETURN::(float[8]));

Cp_Cu  := define_external('cp_cu', LIB=libGfer, 
			 'T'		::REF(float[8]), RETURN::(float[8]));

K_Cu 	 := define_external('k_cu', LIB=libGfer, 
			 'T'			::REF(float[8]),
			 'RRR'		::REF(float[8]), RETURN::(float[8]));

d_sk	 := define_external('d_sk', LIB=libGfer, 
			 'mu_r'		::REF(float[8]),
			 's'			::REF(float[8]),
			 'f'			::REF(float[8]), RETURN::(float[8]));

df_skTE := define_external('df_skTE', LIB=libGfer, 
			 'd_sk'	::REF(float[8]),
			 'Req'	::REF(float[8]),
  		 'df_f'	::REF(float[8]),	
			 'dg_g'	::REF(float[8]));

df_skTM1n := define_external('df_skTM1n', LIB=libGfer, 
			 'd_sk'	::REF(float[8]),
			 'Req'	::REF(float[8]),
			 'n'    ::REF(integer[4]),
  		 'df_f'	::REF(float[8]),	
			 'dg_g'	::REF(float[8]));

df_dcTM1n := define_external('df_dcTM1n', LIB=libGfer, 
			 'r'		::REF(float[8]),
			 'Req'	::REF(float[8]),
			 'n'		::REF(integer[4]), RETURN::(float[8]));

df_dcTE := define_external('df_dcTE', LIB=libGfer, 
			 'r'		::REF(float[8]),
			 'Req'	::REF(float[8]), RETURN::(float[8]));

Eps_TM1n := define_external('Eps_TM1n', LIB=libGfer, 
			 'f1'	::REF(float[8]),
			 'f2'	::REF(float[8]),
			 'f3' ::REF(float[8]),
			 'n'  ::REF(integer[4]),
  		 'ep1'::REF(float[8]),	
			 'ep2'::REF(float[8]));

Eps_TE1n := define_external('Eps_TE1n', LIB=libGfer, 
			 'f1'	::REF(float[8]),
			 'f2'	::REF(float[8]),
			 'f3' ::REF(float[8]),
			 'n'  ::REF(integer[4]),
  		 'ep1'::REF(float[8]),	
			 'ep2'::REF(float[8]));

dzm2_zTM1n := define_external('dzm2_zTM1n', LIB=libGfer, 
			 'ep1'	::REF(float[8]),
			 'ep2'	::REF(float[8]),
			 'n'  ::REF(integer[4]), RETURN::(float[8]));

dzm2_zTE1n := define_external('dzm2_zTE1n', LIB=libGfer, 
			 'ep1'	::REF(float[8]),
			 'ep2'	::REF(float[8]),
			 'n'  	::REF(integer[4]), RETURN::(float[8]));

dz2_z2AC := define_external('O2dZ2AC', LIB=libGfer,
			'Eps'		:: ARRAY(1..n, float[8]),
			'n'     :: REF(integer[4]), RETURN::(float[8]));

end module:

